@@ -88,7 +88,6 @@ def train_from_folder(
     generate_interpolation = False,
     aug_test = False,
     attn_res_layers = [32],
-    sle_spatial = False,
     disc_output_size = 1,
     antialias = False,
     interpolation_num_steps = 100,
@@ -110,7 +109,6 @@ def train_from_folder(
         batch_size = batch_size,
         gradient_accumulate_every = gradient_accumulate_every,
         attn_res_layers = cast_list(attn_res_layers),
-        sle_spatial = sle_spatial,
         disc_output_size = disc_output_size,
         antialias = antialias,
         image_size = image_size,
